#define F_CPU 16000000UL

#define TIMER0_COMPARE 249  // 1ms s 64 prescla pri 16MHz

#include "pitches.h"
#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define set_bit(ADDRESS, BIT) (ADDRESS |= (1 << BIT))
#define clear_bit(ADDRESS, BIT) (ADDRESS &= ~(1 << BIT))
#define toggle_bit(ADDRESS, BIT) (ADDRESS ^= (1 << BIT))

#define buzz1 PB3   // D11
#define buzz2 PB2   // D10
#define course PB1  // D9
#define button PD2  // D2

volatile unsigned long timer0_millis = 0;

ISR(TIMER0_COMPA_vect) {
  timer0_millis++;
}

unsigned long millis() {
  unsigned long ms;
  cli();
  ms = timer0_millis;
  sei();
  return ms;
}

void setup_timer0() {
  TCCR0A |= (1 << WGM01);                         // CTC mode
  TCCR0B |= (1 << CS01) | (1 << CS00);            // Prescaler 64
  OCR0A = TIMER0_COMPARE;
  TIMSK0 |= (1 << OCIE0A);                        // Enable compare interrupt
  sei();                                          // Enable global interrupts
}

// Music
int songState = 0;

int melody[] = {
  NOTE_F4, NOTE_E4, NOTE_D4, NOTE_CS4,
  NOTE_C4, NOTE_B3, NOTE_AS3, NOTE_A3,
  NOTE_G3, NOTE_A3, NOTE_AS3, NOTE_A3,
  NOTE_G3, NOTE_C4, 0,

  NOTE_C4, NOTE_A3, NOTE_A3, NOTE_A3,
  NOTE_GS3, NOTE_A3, NOTE_F4, NOTE_C4,
  NOTE_C4, NOTE_A3, NOTE_AS3, NOTE_AS3,
  NOTE_AS3, NOTE_C4, NOTE_D4, 0,

  NOTE_AS3, NOTE_G3, NOTE_G3, NOTE_G3,
  NOTE_FS3, NOTE_G3, NOTE_E4, NOTE_D4,
  NOTE_D4, NOTE_AS3, NOTE_A3, NOTE_A3,
  NOTE_A3, NOTE_AS3, NOTE_C4, 0,

  NOTE_C4, NOTE_A3, NOTE_A3, NOTE_A3,
  NOTE_GS3, NOTE_A3, NOTE_A4, NOTE_F4,
  NOTE_F4, NOTE_C4, NOTE_B3, NOTE_G4,
  NOTE_G4, NOTE_G4, NOTE_G4, 0,

  NOTE_G4, NOTE_E4, NOTE_G4, NOTE_G4,
  NOTE_FS4, NOTE_G4, NOTE_D4, NOTE_G4,
  NOTE_G4, NOTE_FS4, NOTE_G4, NOTE_C4,
  NOTE_B3, NOTE_C4, NOTE_B3, NOTE_C4, 0
};

int tempo[] = {
  8, 16, 8, 16,
  8, 16, 8, 16,
  16, 16, 16, 8,
  16, 8, 3,

  12, 16, 16, 16,
  8, 16, 8, 16,
  8, 16, 8, 16,
  8, 16, 4, 12,

  12, 16, 16, 16,
  8, 16, 8, 16,
  8, 16, 8, 16,
  8, 16, 4, 12,

  12, 16, 16, 16,
  8, 16, 8, 16,
  8, 16, 8, 16,
  8, 16, 4, 16,

  12, 17, 17, 17,
  8, 12, 17, 17,
  17, 8, 16, 8,
  16, 8, 16, 8, 1
};

#define MELODY_LENGTH (sizeof(melody)/sizeof(melody[0]))

unsigned long previousMillis1 = 0;
const long interval1 = 1500;
unsigned long previousMillis2 = 0;
const long interval2 = 100;
unsigned long previousMillis3 = 0;
const long interval3 = 1000;

int displayStatus = 0;
int mode = 1;
int countdown = 0;
int count = 20;

void setup() {
  setup_timer0();

  // Set buzzers ako vstupi
  set_bit(DDRB, buzz1);
  set_bit(DDRB, buzz2);

  // Set course wire ako vstup s pull-up
  clear_bit(DDRB, course);
  set_bit(PORTB, course);

  // Set button ako vstup s pull-up
  clear_bit(DDRD, button);
  set_bit(PORTD, button);
}

void buzz(int duration, int frequency) {
  if (frequency == 0) return;

  long delayValue = 1138; // priemer aby to znelo OK

  // vypocetm kolko cyclov treba na duration a frequecny
  long numCycles = (long)frequency * duration / 1000;

  for (long i = 0; i < numCycles; i++) {
    set_bit(PORTB, buzz1);  // Turn on buzzer 1
    set_bit(PORTB, buzz2);  // Turn on buzzer 2
    _delay_us(delayValue);  

    clear_bit(PORTB, buzz1); // Turn off buzzer 1
    clear_bit(PORTB, buzz2); // Turn off buzzer 2
    _delay_us(delayValue);   
  }
}

void sing() {
  unsigned long currentMillis = millis();

  if (currentMillis - previousMillis2 >= interval2) {
    previousMillis2 = currentMillis;

    int currentNote = melody[songState];
    int duration = 1000 / tempo[songState];

    buzz(duration, currentNote);  // hraj na oba buzzers
    buzz(10, 0);                  // kratke ticho

    ++songState;

    if (songState > 79) {
      songState = 14;  // Loop 
    }
  }
}

void collision_sound()  {
  for (int i = 0; i < 3; i++) {
    buzz(100, NOTE_E5); // zvuk dotknutia sa
    buzz(100, 0);
  }
}

volatile uint8_t gameRunning = 0;
uint8_t lastButtonState = 1;

void loop() {
  uint8_t currentButtonState = PIND & (1 << button);

  // button stlacenie (rising)
  if (!currentButtonState && lastButtonState) {
    _delay_ms(50);  
    currentButtonState = PIND & (1 << button);
    if (!currentButtonState) {
      gameRunning = !gameRunning;
    }
  }

  lastButtonState = currentButtonState;

  static int wireWasHit = 0;

  if (gameRunning) {
    if (!(PINB & (1 << course))) {  // kabel dotknuty
      if (!wireWasHit) {
        wireWasHit = 1;
        collision_sound();  // hraj koliziu
      }
    } else {
      wireWasHit = 0;  
    }

    sing();  // hrat hudbu ked si zapnuty
  }
}

int main(void) {
  setup();
  while (1) {
    loop();
  }
}